@@ -8,6 +8,14 @@
  * Licensed under GPLv2 or later, see file LICENSE in this source tree.
  */
 
+//usage:#define findfs_trivial_usage
+//usage:       "LABEL=label or UUID=uuid"
+//usage:#define findfs_full_usage "\n\n"
+//usage:       "Find a filesystem device based on a label or UUID"
+//usage:
+//usage:#define findfs_example_usage
+//usage:       "$ findfs LABEL=MyDevice"
+
 #include "libbb.h"
 #include "volume_id.h"
 
@@ -19,7 +27,7 @@ int findfs_main(int argc UNUSED_PARAM, char **argv)
 	if (!dev)
 		bb_show_usage();
 
-	if (strncmp(dev, "/dev/", 5) == 0) {
+	if (is_prefixed_with(dev, "/dev/")) {
 		/* Just pass any /dev/xxx name right through.
 		 * This might aid in some scripts being able
 		 * to call this unconditionally */
