@@ -7,6 +7,11 @@
  *
  * Licensed under GPLv2 or later, see file LICENSE in this source tree.
  */
+
+//kbuild:lib-$(CONFIG_BLKID) += get_devname.o
+//kbuild:lib-$(CONFIG_FINDFS) += get_devname.o
+//kbuild:lib-$(CONFIG_FEATURE_MOUNT_LABEL) += get_devname.o
+
 #include <sys/mount.h> /* BLKGETSIZE64 */
 #if !defined(BLKGETSIZE64)
 # define BLKGETSIZE64 _IOR(0x12,114,size_t)
@@ -49,7 +54,11 @@ get_label_uuid(int fd, char **label, char **uuid, const char **type)
 	if (volume_id_probe_all(vid, /*0,*/ size) != 0)
 		goto ret;
 
-	if (vid->label[0] != '\0' || vid->uuid[0] != '\0') {
+	if (vid->label[0] != '\0' || vid->uuid[0] != '\0'
+#if ENABLE_FEATURE_BLKID_TYPE
+	 || vid->type != NULL
+#endif
+	) {
 		*label = xstrndup(vid->label, sizeof(vid->label));
 		*uuid  = xstrndup(vid->uuid, sizeof(vid->uuid));
 #if ENABLE_FEATURE_BLKID_TYPE
@@ -114,12 +123,12 @@ uuidcache_check_device(const char *device,
 	return TRUE;
 }
 
-static void
-uuidcache_init(void)
+static struct uuidCache_s*
+uuidcache_init(int scan_devices)
 {
 	dbg("DBG: uuidCache=%x, uuidCache");
 	if (uuidCache)
-		return;
+		return uuidCache;
 
 	/* We were scanning /proc/partitions
 	 * and /proc/sys/dev/cdrom/info here.
@@ -131,12 +140,14 @@ uuidcache_init(void)
 	 * This is unacceptably complex. Let's just scan /dev.
 	 * (Maybe add scanning of /sys/block/XXX/dev for devices
 	 * somehow not having their /dev/XXX entries created?) */
+	if (scan_devices)
+		recursive_action("/dev", ACTION_RECURSE,
+			uuidcache_check_device, /* file_action */
+			NULL, /* dir_action */
+			NULL, /* userData */
+			0 /* depth */);
 
-	recursive_action("/dev", ACTION_RECURSE,
-		uuidcache_check_device, /* file_action */
-		NULL, /* dir_action */
-		NULL, /* userData */
-		0 /* depth */);
+	return uuidCache;
 }
 
 #define UUID   1
@@ -148,9 +159,7 @@ get_spec_by_x(int n, const char *t, int *majorPtr, int *minorPtr)
 {
 	struct uuidCache_s *uc;
 
-	uuidcache_init();
-	uc = uuidCache;
-
+	uc = uuidcache_init(/*scan_devices:*/ 1);
 	while (uc) {
 		switch (n) {
 		case UUID:
@@ -215,24 +224,23 @@ get_spec_by_volume_label(const char *s, int *major, int *minor)
 #endif // UNUSED
 
 /* Used by blkid */
-void display_uuid_cache(void)
+void display_uuid_cache(int scan_devices)
 {
-	struct uuidCache_s *u;
+	struct uuidCache_s *uc;
 
-	uuidcache_init();
-	u = uuidCache;
-	while (u) {
-		printf("%s:", u->device);
-		if (u->label[0])
-			printf(" LABEL=\"%s\"", u->label);
-		if (u->uc_uuid[0])
-			printf(" UUID=\"%s\"", u->uc_uuid);
+	uc = uuidcache_init(scan_devices);
+	while (uc) {
+		printf("%s:", uc->device);
+		if (uc->label[0])
+			printf(" LABEL=\"%s\"", uc->label);
+		if (uc->uc_uuid[0])
+			printf(" UUID=\"%s\"", uc->uc_uuid);
 #if ENABLE_FEATURE_BLKID_TYPE
-	if (u->type)
-		printf(" TYPE=\"%s\"", u->type);
+	if (uc->type)
+		printf(" TYPE=\"%s\"", uc->type);
 #endif
 		bb_putchar('\n');
-		u = u->next;
+		uc = uc->next;
 	}
 }
 
@@ -265,8 +273,7 @@ char *get_devname_from_label(const char *spec)
 {
 	struct uuidCache_s *uc;
 
-	uuidcache_init();
-	uc = uuidCache;
+	uc = uuidcache_init(/*scan_devices:*/ 1);
 	while (uc) {
 		if (uc->label[0] && strcmp(spec, uc->label) == 0) {
 			return xstrdup(uc->device);
@@ -280,8 +287,7 @@ char *get_devname_from_uuid(const char *spec)
 {
 	struct uuidCache_s *uc;
 
-	uuidcache_init();
-	uc = uuidCache;
+	uc = uuidcache_init(/*scan_devices:*/ 1);
 	while (uc) {
 		/* case of hex numbers doesn't matter */
 		if (strcasecmp(spec, uc->uc_uuid) == 0) {
@@ -296,9 +302,9 @@ int resolve_mount_spec(char **fsname)
 {
 	char *tmp = *fsname;
 
-	if (strncmp(*fsname, "UUID=", 5) == 0)
+	if (is_prefixed_with(*fsname, "UUID="))
 		tmp = get_devname_from_uuid(*fsname + 5);
-	else if (strncmp(*fsname, "LABEL=", 6) == 0)
+	else if (is_prefixed_with(*fsname, "LABEL="))
 		tmp = get_devname_from_label(*fsname + 6);
 
 	if (tmp == *fsname)
